@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders.c,v 1.146 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders.c,v 1.146 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -191,8 +191,14 @@ genders_load_data(genders_t handle, const char *filename)
 
   if (_genders_open_and_parse(handle, 
 			      filename, 
+			      &handle->numattrs,
+			      &handle->maxattrs,
+			      &handle->maxnodelen,
+			      &handle->maxattrlen,
+			      &handle->maxvallen,
 			      handle->nodeslist, 
 			      handle->attrvalslist,
+			      handle->attrslist,
                               &(handle->node_index),
                               &(handle->node_index_size),
                               &(handle->attr_index),
@@ -588,6 +594,12 @@ genders_getnodes(genders_t handle, char *nodes[], int len,
       goto cleanup;
     }
 
+  if (attr && !strlen(attr))
+    attr = NULL;
+
+  if (val && !strlen(val))
+    val = NULL;
+
   if (handle->attrval_index 
       && attr
       && val
@@ -669,7 +681,7 @@ genders_getattr(genders_t handle,
 {
   ListIterator attrlist_itr = NULL;
   ListIterator attrvals_itr = NULL;
-  List attrvals;
+  genders_attrvals_container_t avc;
   genders_node_t n;
   int index = 0, rv = -1;
 
@@ -682,7 +694,7 @@ genders_getattr(genders_t handle,
       goto cleanup;
     }
 
-  if (!node)
+  if (!node || !strlen(node))
     node = handle->nodename;
   
   if (!handle->numnodes)
@@ -698,11 +710,11 @@ genders_getattr(genders_t handle,
     }
 
   __list_iterator_create(attrlist_itr, n->attrlist);
-  while ((attrvals = list_next(attrlist_itr))) 
+  while ((avc = list_next(attrlist_itr))) 
     {
       genders_attrval_t av;
       
-      __list_iterator_create(attrvals_itr, attrvals);
+      __list_iterator_create(attrvals_itr, avc->attrvals);
       while ((av = list_next(attrvals_itr))) 
 	{
 	  if (_genders_put_in_array(handle, av->attr, attrs, index, len) < 0)
@@ -779,13 +791,15 @@ genders_testattr(genders_t handle,
   if (_genders_loaded_handle_error_check(handle) < 0)
     return -1;
 
-  if (!attr || (val && len < 0)) 
+  if (!attr
+      || !strlen(attr)
+      || (val && len < 0)) 
     {
       handle->errnum = GENDERS_ERR_PARAMETERS;
       return -1;
     }
 
-  if (!node)
+  if (!node || !strlen(node))
     node = handle->nodename;
 
   if (!handle->numnodes)
@@ -812,7 +826,7 @@ genders_testattr(genders_t handle,
 	      char *valptr;
 	      if (_genders_get_valptr(handle, n, av, &valptr, NULL) < 0)
 		return -1;
-	      if (strlen(valptr + 1) > len) 
+	      if ((strlen(valptr) + 1) > len) 
 		{
 		  handle->errnum = GENDERS_ERR_OVERFLOW;
 		  return -1;
@@ -840,15 +854,18 @@ genders_testattrval(genders_t handle,
   if (_genders_loaded_handle_error_check(handle) < 0)
     return -1;
 
-  if (!attr) 
+  if (!attr || !strlen(attr)) 
     {
       handle->errnum = GENDERS_ERR_PARAMETERS;
       return -1;
     }
 
-  if (!node)
+  if (!node || !strlen(node))
     node = handle->nodename;
 
+  if (val && !strlen(val))
+    val = NULL;
+
   if (!handle->numnodes)
     {
       handle->errnum = GENDERS_ERR_NOTFOUND;
@@ -876,7 +893,7 @@ genders_isnode(genders_t handle, const char *node)
   if (_genders_loaded_handle_error_check(handle) < 0)
     return -1;
 
-  if (!node)
+  if (!node || !strlen(node))
     node = handle->nodename;
 
   if (!handle->numnodes)
@@ -899,7 +916,7 @@ genders_isattr(genders_t handle, const char *attr)
   if (_genders_loaded_handle_error_check(handle) < 0)
     return -1;
 
-  if (!attr) 
+  if (!attr || !strlen(attr)) 
     {
       handle->errnum = GENDERS_ERR_PARAMETERS;
       return -1;
@@ -928,7 +945,10 @@ genders_isattrval(genders_t handle, const char *attr, const char *val)
   if (_genders_loaded_handle_error_check(handle) < 0)
     goto cleanup;
 
-  if (!attr || !val) 
+  if (!attr
+      || !strlen(attr)
+      || !val
+      || !strlen(val))
     {
       handle->errnum = GENDERS_ERR_PARAMETERS;
       goto cleanup;
@@ -992,7 +1012,7 @@ genders_index_attrvals(genders_t handle, const char *attr)
   if (_genders_loaded_handle_error_check(handle) < 0)
     return -1;
 
-  if (!attr) 
+  if (!attr || !strlen(attr))
     {
       handle->errnum = GENDERS_ERR_PARAMETERS;
       goto cleanup;
@@ -1114,8 +1134,14 @@ int
 genders_parse(genders_t handle, const char *filename, FILE *stream) 
 {
   int errcount, rv = -1;
+  int debugnumattrs = 0;
+  int debugmaxattrs = 0;
+  int debugmaxnodelen = 0;
+  int debugmaxattrlen = 0;
+  int debugmaxvallen = 0;
   List debugnodeslist = NULL;
   List debugattrvalslist = NULL;
+  List debugattrslist = NULL;
   hash_t debugnode_index = NULL;
   int debugnode_index_size = GENDERS_NODE_INDEX_INIT_SIZE;
   hash_t debugattr_index = NULL;
@@ -1128,7 +1154,8 @@ genders_parse(genders_t handle, const char *filename, FILE *stream)
     stream = stderr;
 
   __list_create(debugnodeslist, _genders_list_free_genders_node);
-  __list_create(debugattrvalslist, NULL);
+  __list_create(debugattrvalslist, _genders_list_free_attrvallist);
+  __list_create(debugattrslist, free);
   __hash_create(debugnode_index,
                 debugnode_index_size,
                 (hash_key_f)hash_key_string,
@@ -1142,8 +1169,14 @@ genders_parse(genders_t handle, const char *filename, FILE *stream)
 
   if ((errcount = _genders_open_and_parse(handle, 
 					  filename,
+					  &debugnumattrs,
+					  &debugmaxattrs,
+					  &debugmaxnodelen,
+					  &debugmaxattrlen,
+					  &debugmaxvallen,
 					  debugnodeslist, 
 					  debugattrvalslist,
+					  debugattrslist,
                                           &(debugnode_index),
                                           &(debugnode_index_size),
                                           &(debugattr_index),
@@ -1157,6 +1190,7 @@ genders_parse(genders_t handle, const char *filename, FILE *stream)
  cleanup:
   __list_destroy(debugnodeslist);
   __list_destroy(debugattrvalslist);
+  __list_destroy(debugattrslist);
   __hash_destroy(debugnode_index);
   __hash_destroy(debugattr_index);
   return rv;
@@ -1170,3 +1204,411 @@ genders_set_errnum(genders_t handle, int errnum)
 
   handle->errnum = errnum;
 }
+
+/* 
+ * _genders_copy_nodeslist
+ *
+ * Copy contents of the nodeslist list into the handlecopy.
+ *
+ */
+static int
+_genders_copy_nodeslist(genders_t handle, genders_t handlecopy)
+{
+  ListIterator itr = NULL;
+  genders_node_t n = NULL;
+  genders_node_t newn = NULL;
+  int rv = -1;
+
+  __list_iterator_create(itr, handle->nodeslist);
+  while ((n = list_next(itr)))
+    {
+      __xmalloc(newn, genders_node_t, sizeof(struct genders_node));
+      __xstrdup(newn->name, n->name);
+      __list_create(newn->attrlist, NULL);
+      newn->attrcount = n->attrcount;
+      newn->attrlist_index_size = n->attrlist_index_size;
+      __hash_create(newn->attrlist_index,
+                    newn->attrlist_index_size,
+                    (hash_key_f)hash_key_string,
+                    (hash_cmp_f)strcmp,
+                    NULL);
+
+      __list_append(handlecopy->nodeslist, newn);
+      newn = NULL;
+    }
+  
+  rv = 0;
+ cleanup:
+  if (rv < 0)
+    {
+      if (newn)
+	{
+	  free(newn->name);
+	  __list_destroy(newn->attrlist);
+	  __hash_destroy(newn->attrlist_index);
+	  free(newn);
+	}
+    }
+  __list_iterator_destroy(itr);
+  return rv;
+}
+
+/* 
+ * _genders_copy_fill_node_index
+ *
+ * Add keys into the node_index
+ *
+ */
+static int _genders_copy_fill_node_index(genders_t handle, genders_t handlecopy)
+{
+  List l = NULL;
+  ListIterator itr = NULL;
+  genders_node_t n;
+  int rv = -1;
+
+  __list_iterator_create(itr, handlecopy->nodeslist);
+  while ((n = list_next(itr)))
+    __hash_insert(handlecopy->node_index, n->name, n);
+  
+  rv = 0;
+ cleanup:
+  if (rv < 0)
+    __list_destroy(l);
+  __list_iterator_destroy(itr);
+  return rv;
+}
+
+/* 
+ * _genders_copy_attrvalslist
+ *
+ * Copy contents of the attrvalslist list into the handlecopy.
+ *
+ */
+static int
+_genders_copy_attrvalslist(genders_t handle, genders_t handlecopy)
+{
+  ListIterator attrvalslistitr = NULL;
+  ListIterator attrvalsitr = NULL;
+  genders_attrvals_container_t avc;
+  genders_attrvals_container_t newavc = NULL;
+  genders_attrval_t newav = NULL;
+  int rv = -1;
+
+  __list_iterator_create(attrvalslistitr, handle->attrvalslist);
+  while ((avc = list_next(attrvalslistitr)))
+    {
+      genders_attrval_t av = NULL;
+
+      __list_iterator_create(attrvalsitr, avc->attrvals);
+
+      __xmalloc(newavc,
+		genders_attrvals_container_t,
+		sizeof(struct genders_attrvals_container));
+      __list_create(newavc->attrvals, _genders_list_free_genders_attrval);
+      newavc->index = avc->index;
+
+      while ((av = list_next(attrvalsitr)))
+	{
+	  __xmalloc(newav, genders_attrval_t, sizeof(struct genders_attrval));
+	  __xstrdup(newav->attr, av->attr);
+	  if (av->val)
+	    __xstrdup(newav->val, av->val);
+	  else
+	    newav->val = NULL;
+	  newav->val_contains_subst = av->val_contains_subst;
+	  __list_append(newavc->attrvals, newav);
+	  newav = NULL;
+	}
+      
+      __list_append(handlecopy->attrvalslist, newavc);
+      newavc = NULL;
+  }
+  
+  rv = 0;
+ cleanup:
+  if (rv < 0)
+    {
+      if (newav)
+	{
+	  free(newav->attr);
+	  free(newav->val);
+	  free(newav);
+	}
+      if (newavc)
+	{
+	  __list_destroy(newavc->attrvals);
+	  free(newavc);
+	}
+    }
+  __list_iterator_destroy(attrvalslistitr);
+  __list_iterator_destroy(attrvalsitr);
+  return rv;
+}
+
+/* 
+ * _genders_copy_attrslist
+ *
+ * Copy contents of the attrslist list into the handlecopy.
+ *
+ */
+static int
+_genders_copy_attrslist(genders_t handle, genders_t handlecopy)
+{
+  ListIterator itr = NULL;
+  char *newattr = NULL;
+  char *attr;
+  int rv = -1;
+
+  __list_iterator_create(itr, handle->attrslist);
+  while ((attr = list_next(itr)))
+    {
+      __xstrdup(newattr, attr);
+      __list_append(handlecopy->attrslist, newattr);
+      newattr = NULL;
+    }
+  
+  rv = 0;
+ cleanup:
+  if (rv < 0)
+    free(newattr);
+  __list_iterator_destroy(itr);
+  return rv;
+}
+
+/* 
+ * _genders_copy_fill_attr_index
+ *
+ * Add keys into the attr_index
+ *
+ */
+static int _genders_copy_fill_attr_index(genders_t handle, genders_t handlecopy)
+{
+  List l = NULL;
+  ListIterator itr = NULL;
+  char *attr;
+  int rv = -1;
+
+  __list_iterator_create(itr, handlecopy->attrslist);
+  while ((attr = list_next(itr)))
+    {
+      __list_create(l, NULL);
+      __hash_insert(handlecopy->attr_index, attr, l);
+      l = NULL;
+    }
+
+  rv = 0;
+ cleanup:
+  if (rv < 0)
+    __list_destroy(l);
+  __list_iterator_destroy(itr);
+  return rv;
+}
+
+/*
+ * _genders_copy_find_attrvals_container
+ *
+ * Find the genders_attrvals_container pointer indicated by the index.
+ */
+static genders_attrvals_container_t
+_genders_copy_find_attrvals_container(genders_t handle,
+				      genders_t handlecopy,
+				      unsigned int index)
+{
+  ListIterator attrvalsitr = NULL;
+  genders_attrvals_container_t rv = NULL;
+  genders_attrvals_container_t avc;
+  
+  __list_iterator_create(attrvalsitr, handlecopy->attrvalslist);
+  while ((avc = list_next(attrvalsitr)))
+    {
+      if (avc->index == index)
+	{
+	  rv = avc;
+	  goto cleanup;
+	}
+    }
+
+ cleanup:
+  __list_iterator_destroy(attrvalsitr);
+  return rv;
+}
+
+/* 
+ * _genders_copy_fill_node
+ *
+ * Fill node attrlist and attrlist index.
+ *
+ */
+static int
+_genders_copy_fill_node(genders_t handle,
+			genders_t handlecopy,
+			genders_node_t nodehandle,
+			genders_node_t nodecopy)
+{
+  ListIterator attrlistitr = NULL;
+  ListIterator attrvalsitr = NULL;
+  genders_attrvals_container_t avc;
+  int rv = -1;
+
+  __list_iterator_create(attrlistitr, nodehandle->attrlist);
+  while ((avc = list_next(attrlistitr)))
+    {
+      genders_attrvals_container_t tmpavc;
+      genders_attrval_t av;
+
+      if (!(tmpavc = _genders_copy_find_attrvals_container(handle,
+							   handlecopy,
+							   avc->index)))
+	goto cleanup;
+      
+      __list_append(nodecopy->attrlist, tmpavc);
+      
+      __list_iterator_create(attrvalsitr, tmpavc->attrvals);
+      while ((av = list_next(attrvalsitr)))
+	{
+	  List l;
+
+	  __hash_insert(nodecopy->attrlist_index,
+			av->attr,
+			tmpavc);
+	  
+	  if (!(l = hash_find(handlecopy->attr_index, av->attr)))
+	    {
+	      handle->errnum = GENDERS_ERR_INTERNAL;
+	      goto cleanup;
+	    }
+	  
+	  __list_append(l, nodecopy);
+	}
+      
+      __list_iterator_destroy(attrvalsitr);
+      attrvalsitr = NULL;
+    }
+  
+  rv = 0;
+ cleanup:
+  __list_iterator_destroy(attrlistitr);
+  __list_iterator_destroy(attrvalsitr);
+  return rv;
+}
+
+/* 
+ * _genders_copy_fill_node_data
+ *
+ * Fill node attrlist and attrlist index.
+ *
+ */
+static int
+_genders_copy_fill_node_data(genders_t handle, genders_t handlecopy)
+{
+  ListIterator nodeslistitr = NULL;
+  genders_node_t nodecopy;
+  int rv = -1;
+
+  __list_iterator_create(nodeslistitr, handlecopy->nodeslist);
+  while ((nodecopy = list_next(nodeslistitr)))
+    {
+      genders_node_t nodehandle;
+
+      if (!(nodehandle = hash_find(handle->node_index, nodecopy->name)))
+	{
+	  /* Shouldn't be possible to error here */
+	  handle->errnum = GENDERS_ERR_INTERNAL;
+	  goto cleanup;
+	}
+
+      if (_genders_copy_fill_node(handle,
+				  handlecopy,
+				  nodehandle,
+				  nodecopy) < 0)
+	goto cleanup;
+    }
+  
+  rv = 0;
+ cleanup:
+  __list_iterator_destroy(nodeslistitr);
+  return rv;
+}
+
+genders_t
+genders_copy(genders_t handle) 
+{
+  genders_t handlecopy = NULL;
+
+  if (_genders_loaded_handle_error_check(handle) < 0)
+    return NULL;
+
+  if (!(handlecopy = genders_handle_create()))
+    {
+      handle->errnum = GENDERS_ERR_OUTMEM;
+      goto cleanup;
+    }
+
+  handlecopy->is_loaded = handle->is_loaded;
+  handlecopy->numnodes = handle->numnodes;
+  handlecopy->numattrs = handle->numattrs;
+  handlecopy->maxattrs = handle->maxattrs;
+  handlecopy->maxnodelen = handle->maxnodelen;
+  handlecopy->maxattrlen = handle->maxattrlen;
+  handlecopy->maxvallen = handle->maxvallen;
+
+  memcpy(handlecopy->nodename, handle->nodename, GENDERS_MAXHOSTNAMELEN+1);
+  
+  if (_genders_copy_nodeslist(handle, handlecopy) < 0)
+    goto cleanup;
+
+  handlecopy->node_index_size = handle->node_index_size;
+
+  __hash_create(handlecopy->node_index,
+                handlecopy->node_index_size,
+                (hash_key_f)hash_key_string,
+                (hash_cmp_f)strcmp,
+                NULL);
+
+  if (_genders_copy_fill_node_index(handle, handlecopy) < 0)
+    goto cleanup;
+
+  if (_genders_copy_attrvalslist(handle, handlecopy) < 0)
+    goto cleanup;
+
+  if (_genders_copy_attrslist(handle, handlecopy) < 0)
+    goto cleanup;
+
+  handlecopy->attr_index_size = handle->attr_index_size;
+  
+  __hash_create(handlecopy->attr_index,
+                handlecopy->attr_index_size,
+                (hash_key_f)hash_key_string,
+                (hash_cmp_f)strcmp,
+                (hash_del_f)list_destroy);
+
+  if (_genders_copy_fill_attr_index(handle, handlecopy) < 0)
+    goto cleanup;
+  
+  /* XXX fill each node with attrvalslist and attr_index */
+  if (_genders_copy_fill_node_data(handle, handlecopy) < 0)
+    goto cleanup;
+
+  /* Create a buffer for value substitutions */
+  __xmalloc(handlecopy->valbuf, char *, handlecopy->maxvallen + 1);
+
+  /* attrval_index, attrval_index_attr, and attrval_buflist
+   * set/re-created by genders_index_attrvals
+   */
+  if (handle->attrval_index)
+    {
+      if (genders_index_attrvals(handlecopy, handle->attrval_index_attr) < 0)
+	{
+	  handle->errnum = GENDERS_ERR_INTERNAL;
+	  goto cleanup;
+	}
+    }
+
+  handle->errnum = GENDERS_ERR_SUCCESS;
+  return handlecopy;
+
+ cleanup:
+  if (handlecopy)
+    (void)genders_handle_destroy(handlecopy);
+  return NULL;
+}
