@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders.h.in,v 1.39 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders.h.in,v 1.39 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2003 The Regents of the University of California.
@@ -27,6 +27,10 @@
 #ifndef _GENDERS_H
 #define _GENDERS_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #include <stdio.h>
 
 #define GENDERS_ERR_SUCCESS       0
@@ -414,4 +418,17 @@ int genders_parse(genders_t handle, const char *filename, FILE *stream);
  */      
 void genders_set_errnum(genders_t handle, int errnum);
 
+/*
+ * genders_copy
+ *
+ * Creates and returns a copy of a loaded genders handle.
+ *
+ * Returns new genders handle on success, NULL on error.
+ */
+genders_t genders_copy(genders_t handle);
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* _GENDERS_H */
