@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_api.h,v 1.8 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders_api.h,v 1.8 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -73,6 +73,19 @@ struct genders_attrval {
 };
 typedef struct genders_attrval *genders_attrval_t;
 
+/*
+ * struct genders_attrvals_container
+ *
+ * stores attrvals list and an index indicating its location.
+ * Implemented predominantly to make genders_copy() easier to
+ * implement.
+ */
+struct genders_attrvals_container {
+  List attrvals;
+  unsigned int index;
+};
+typedef struct genders_attrvals_container *genders_attrvals_container_t;
+
 /* 
  * struct genders
  * 
