@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_parsing.h,v 1.10 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders_parsing.h,v 1.10 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -44,8 +44,14 @@
  */
 int _genders_open_and_parse(genders_t handle,
 			    const char *filename,
+			    int *numattrs,
+			    int *maxattrs,
+			    int *maxnodelen,
+			    int *maxattrlen,
+			    int *maxvallen,
 			    List nodeslist,
 			    List attrvalslist,
+			    List attrslist,
                             hash_t *node_index,
                             int *node_index_size,
                             hash_t *attr_index,
