@@ -91,7 +91,7 @@
 #line 1 "./genders_query.y"
 
 /*****************************************************************************\
- *  $Id: genders_query.y,v 1.34 2009/06/02 18:05:21 chu11 Exp $
+ *  $Id: genders_query.y,v 1.34 2009-06-02 18:05:21 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -663,8 +663,8 @@ genders_query(genders_t handle, char *nodes[], int len, const char *query)
       goto cleanup;
     }
 
-  /* Special case for NULL query */
-  if (!query)
+  /* Special case for NULL or empty string query */
+  if (!query || !strlen(query))
     return genders_getnodes(handle, nodes, len, NULL, NULL);
 
   if (_parse_query(handle, query) < 0)
@@ -711,13 +711,13 @@ genders_testquery(genders_t handle,
   if (_genders_loaded_handle_error_check(handle) < 0)
     return -1;
 
-  if (!query)
+  if (!query || !strlen(query))
     {
       handle->errnum = GENDERS_ERR_PARAMETERS;
       return -1;
     }
 
-  if (!node)
+  if (!node || !strlen(node))
     node = handle->nodename;
 
   if (!handle->numnodes)
