@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_test.c,v 1.10 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders_test.c,v 1.10 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -155,6 +155,7 @@ _test_corner_cases(void)
   errtotal += _corner_case(genders_testquery_corner_case, "genders_testquery"); 
   errtotal += _corner_case(genders_parse_corner_case, "genders_parse");
   errtotal += _corner_case(genders_set_errnum_corner_case, "genders_set_errnum");
+  errtotal += _corner_case(genders_copy_corner_case, "genders_copy");
 
   return errtotal;
 }
@@ -222,6 +223,7 @@ _test_functionality(void)
   errtotal += _functionality(genders_testquery_functionality, "genders_testquery");
   errtotal += _functionality(genders_parse_functionality, "genders_parse");
   errtotal += _functionality(genders_set_errnum_functionality, "genders_set_errnum");
+  errtotal += _functionality(genders_copy_functionality, "genders_copy");
 
   return errtotal;
 }
