@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_test_corner_case.c,v 1.14 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders_test_corner_case.c,v 1.14 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -145,6 +145,9 @@ genders_filename_get(genders_filename_type_t type)
     case GENDERS_FILENAME_NULL:
       filename = NULL;
       break;
+    case GENDERS_FILENAME_NON_NULL_EMPTY:
+      filename = "";
+      break;
     case GENDERS_FILENAME_NON_NULL_EXIST:
       filename = genders_database_corner_case.filename;
       break;
@@ -1009,8 +1012,21 @@ genders_getnodes_corner_case(int verbose)
       handle = genders_handle_get(tests[i].param1);
       listptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : list;
       len = (tests[i].param3 == GENDERS_LENGTH_POSITIVE_LARGE) ? list_len : tests[i].param3;
-      attrptr = (tests[i].param4 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->attr_with_val;
-      valptr = (tests[i].param5 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->val;;
+
+      if (tests[i].param4 == GENDERS_STRING_NULL)
+	attrptr = NULL;
+      else if (tests[i].param4 == GENDERS_STRING_NON_NULL_EMPTY)
+	attrptr = "";
+      else
+	attrptr = genders_database_corner_case.data->attr_with_val;
+
+      if (tests[i].param5 == GENDERS_STRING_NULL)
+	valptr = NULL;
+      else if (tests[i].param5 == GENDERS_STRING_NON_NULL_EMPTY)
+	valptr = "";
+      else
+	valptr = genders_database_corner_case.data->val;
+
       return_value = genders_getnodes(handle, listptr, len, attrptr, valptr);
       errnum = genders_errnum(handle);
 
@@ -1063,7 +1079,14 @@ genders_getattr_corner_case(int verbose)
       attrlistptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : attrlist;
       vallistptr = (tests[i].param3 == GENDERS_POINTER_NULL) ? NULL : vallist;
       len = (tests[i].param4 == GENDERS_LENGTH_POSITIVE_LARGE) ? attrlist_len : tests[i].param4;
-      nodeptr = (tests[i].param5 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->node;
+
+      if (tests[i].param5 == GENDERS_STRING_NULL)
+	nodeptr = NULL;
+      else if (tests[i].param5 == GENDERS_STRING_NON_NULL_EMPTY)
+	nodeptr = "";
+      else
+	nodeptr = genders_database_corner_case.data->node;
+
       return_value = genders_getattr(handle, attrlistptr, vallistptr, len, nodeptr);
       errnum = genders_errnum(handle);
 
@@ -1160,8 +1183,21 @@ genders_testattr_corner_case(int verbose)
       int return_value, errnum, len;
 
       handle = genders_handle_get(tests[i].param1);
-      nodeptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->node;
-      attrptr = (tests[i].param3 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->attr_with_val;
+
+      if (tests[i].param2 == GENDERS_STRING_NULL)
+	nodeptr = NULL;
+      else if (tests[i].param2 == GENDERS_STRING_NON_NULL_EMPTY)
+	nodeptr = "";
+      else
+	nodeptr = genders_database_corner_case.data->node;
+
+      if (tests[i].param3 == GENDERS_STRING_NULL)
+	attrptr = NULL;
+      else if (tests[i].param3 == GENDERS_STRING_NON_NULL_EMPTY)
+	attrptr = "";
+      else
+	attrptr = genders_database_corner_case.data->attr_with_val;
+
       valptr = (tests[i].param4 == GENDERS_POINTER_NULL) ? NULL: valbuf;
       len = (tests[i].param5 == GENDERS_LENGTH_POSITIVE_LARGE) ? maxvallen + 1: tests[i].param5;
       return_value = genders_testattr(handle, nodeptr, attrptr, valptr, len);
@@ -1201,9 +1237,28 @@ genders_testattrval_corner_case(int verbose)
       int return_value, errnum;
 
       handle = genders_handle_get(tests[i].param1);
-      nodeptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->node;
-      attrptr = (tests[i].param3 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->attr_with_val;
-      valptr = (tests[i].param4 == GENDERS_POINTER_NULL) ? NULL: genders_database_corner_case.data->val;;
+
+      if (tests[i].param2 == GENDERS_STRING_NULL)
+	nodeptr = NULL;
+      else if (tests[i].param2 == GENDERS_STRING_NON_NULL_EMPTY)
+	nodeptr = "";
+      else
+	nodeptr = genders_database_corner_case.data->node;
+
+      if (tests[i].param3 == GENDERS_STRING_NULL)
+	attrptr = NULL;
+      else if (tests[i].param3 == GENDERS_STRING_NON_NULL_EMPTY)
+	attrptr = "";
+      else
+	attrptr = genders_database_corner_case.data->attr_with_val;
+
+      if (tests[i].param4 == GENDERS_STRING_NULL)
+	valptr = NULL;
+      else if (tests[i].param4 == GENDERS_STRING_NON_NULL_EMPTY)
+	valptr = "";
+      else
+	valptr = genders_database_corner_case.data->val;
+
       return_value = genders_testattrval(handle, nodeptr, attrptr, valptr);
       errnum = genders_errnum(handle);
 
@@ -1237,7 +1292,14 @@ genders_isnode_corner_case(int verbose)
       int return_value, errnum;
 
       handle = genders_handle_get(tests[i].param1);
-      nodeptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->node;
+
+      if (tests[i].param2 == GENDERS_STRING_NULL)
+	nodeptr = NULL;
+      else if (tests[i].param2 == GENDERS_STRING_NON_NULL_EMPTY)
+	nodeptr = "";
+      else
+	nodeptr = genders_database_corner_case.data->node;
+      
       return_value = genders_isnode(handle, nodeptr);
       errnum = genders_errnum(handle);
 
@@ -1271,7 +1333,14 @@ genders_isattr_corner_case(int verbose)
       int return_value, errnum;
 
       handle = genders_handle_get(tests[i].param1);
-      attrptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->attr_without_val;
+
+      if (tests[i].param2 == GENDERS_STRING_NULL)
+	attrptr = NULL;
+      else if (tests[i].param2 == GENDERS_STRING_NON_NULL_EMPTY)
+	attrptr = "";
+      else
+	attrptr = genders_database_corner_case.data->attr_without_val;
+
       return_value = genders_isattr(handle, attrptr);
       errnum = genders_errnum(handle);
 
@@ -1305,8 +1374,21 @@ genders_isattrval_corner_case(int verbose)
       int return_value, errnum;
 
       handle = genders_handle_get(tests[i].param1);
-      attrptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->attr_with_val;
-      valptr = (tests[i].param3 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->val;
+
+      if (tests[i].param2 == GENDERS_STRING_NULL)
+	attrptr = NULL;
+      else if (tests[i].param2 == GENDERS_STRING_NON_NULL_EMPTY)
+	attrptr = "";
+      else
+	attrptr = genders_database_corner_case.data->attr_with_val;
+
+      if (tests[i].param3 == GENDERS_STRING_NULL)
+	valptr = NULL;
+      else if (tests[i].param3 == GENDERS_STRING_NON_NULL_EMPTY)
+	valptr = "";
+      else
+	valptr = genders_database_corner_case.data->val;
+
       return_value = genders_isattrval(handle, attrptr, valptr);
       errnum = genders_errnum(handle);
 
@@ -1340,7 +1422,14 @@ genders_index_attrvals_corner_case(int verbose)
       int return_value, errnum;
 
       handle = genders_handle_get(tests[i].param1);
-      attrptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->attr_with_val;
+
+      if (tests[i].param2 == GENDERS_STRING_NULL)
+	attrptr = NULL;
+      else if (tests[i].param2 == GENDERS_STRING_NON_NULL_EMPTY)
+	attrptr = "";
+      else
+	attrptr = genders_database_corner_case.data->attr_with_val;
+
       return_value = genders_index_attrvals(handle, attrptr);
       errnum = genders_errnum(handle);
 
@@ -1384,7 +1473,12 @@ genders_query_corner_case(int verbose)
       handle = genders_handle_get(tests[i].param1);
       listptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : list;
       len = (tests[i].param3 == GENDERS_LENGTH_POSITIVE_LARGE) ? list_len : tests[i].param3;
-      queryptr = (tests[i].param4 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->attr_with_val;
+      if (tests[i].param4 == GENDERS_STRING_NULL)
+	queryptr = NULL;
+      else if (tests[i].param4 == GENDERS_STRING_NON_NULL_EMPTY)
+	queryptr = "";
+      else
+	queryptr = genders_database_corner_case.data->attr_with_val;
       return_value = genders_query(handle, listptr, len, queryptr);
       errnum = genders_errnum(handle);
 
@@ -1423,11 +1517,24 @@ genders_testquery_corner_case(int verbose)
       int return_value, errnum;
 
       handle = genders_handle_get(tests[i].param1);
-      nodeptr = (tests[i].param2 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->node;
-      queryptr = (tests[i].param3 == GENDERS_POINTER_NULL) ? NULL : genders_database_corner_case.data->attr_with_val;
+
+      if (tests[i].param2 == GENDERS_STRING_NULL)
+	nodeptr = NULL;
+      else if (tests[i].param2 == GENDERS_STRING_NON_NULL_EMPTY)
+	nodeptr = "";
+      else
+	nodeptr = genders_database_corner_case.data->node;
+
+      if (tests[i].param3 == GENDERS_STRING_NULL)
+	queryptr = NULL;
+      else if (tests[i].param3 == GENDERS_STRING_NON_NULL_EMPTY)
+	queryptr = "";
+      else
+	queryptr = genders_database_corner_case.data->attr_with_val;
+
       return_value = genders_testquery(handle, nodeptr, queryptr);
       errnum = genders_errnum(handle);
-
+      
       errcount += genders_return_value_errnum_check("genders_testquery",
 						    tests[i].num,
 						    tests[i].expected_return_value,
@@ -1436,7 +1543,7 @@ genders_testquery_corner_case(int verbose)
 						    errnum,
 						    NULL,
 						    verbose);
-
+      
       genders_handle_cleanup(handle);
       i++;
     }
@@ -1543,3 +1650,36 @@ genders_set_errnum_corner_case(int verbose)
 
   return errcount;
 }
+
+int
+genders_copy_corner_case(int verbose)
+{
+  int i = 0;
+  int errcount = 0;
+  genders_copy_corner_case_t *tests = &genders_copy_corner_case_tests[0];
+
+  while (!(tests[i].num < 0)) 
+    {
+      genders_t handle;
+      genders_t return_value;
+      int errnum;
+
+      handle = genders_handle_get(tests[i].param1);
+      return_value = genders_copy(handle);
+      errnum = genders_errnum(handle);
+ 
+      errcount += genders_return_value_pointer_errnum_check("genders_copy",
+							    tests[i].num,
+							    tests[i].expected_return_value,
+							    tests[i].expected_errnum,
+							    return_value,
+							    errnum,
+							    NULL,
+							    verbose);
+      
+      genders_handle_cleanup(handle);
+      i++;
+    }
+
+  return errcount;
+}
