@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_test_corner_case.h,v 1.8 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders_test_corner_case.h,v 1.8 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -45,6 +45,14 @@ typedef struct {
   int expected_errnum;
 } genders_handle_pointer_corner_case_t;
 
+typedef struct {
+  int num;
+  genders_handle_type_t param1;
+  genders_string_type_t param2;
+  int expected_return_value;
+  int expected_errnum;
+} genders_handle_string_corner_case_t;
+
 typedef struct {
   int num;
   genders_handle_type_t param1;
@@ -110,8 +118,8 @@ typedef struct {
   genders_handle_type_t param1;
   genders_pointer_type_t param2;
   genders_length_type_t param3;
-  genders_pointer_type_t param4;
-  genders_pointer_type_t param5;
+  genders_string_type_t param4;
+  genders_string_type_t param5;
   int expected_return_value;
   int expected_errnum;
 } genders_getnodes_corner_case_t;
@@ -122,7 +130,7 @@ typedef struct {
   genders_pointer_type_t param2;
   genders_pointer_type_t param3;
   genders_length_type_t param4;
-  genders_pointer_type_t param5;
+  genders_string_type_t param5;
   int expected_return_value;
   int expected_errnum;
 } genders_getattr_corner_case_t;
@@ -132,8 +140,8 @@ typedef genders_handle_pointer_length_corner_case_t genders_getattr_all_corner_c
 typedef struct {
   int num;
   genders_handle_type_t param1;
-  genders_pointer_type_t param2;
-  genders_pointer_type_t param3;
+  genders_string_type_t param2;
+  genders_string_type_t param3;
   genders_pointer_type_t param4;
   genders_length_type_t param5;
   int expected_return_value;
@@ -143,33 +151,33 @@ typedef struct {
 typedef struct {
   int num;
   genders_handle_type_t param1;
-  genders_pointer_type_t param2;
-  genders_pointer_type_t param3;
-  genders_pointer_type_t param4;
+  genders_string_type_t param2;
+  genders_string_type_t param3;
+  genders_string_type_t param4;
   int expected_return_value;
   int expected_errnum;
 } genders_testattrval_corner_case_t;
 
-typedef genders_handle_pointer_corner_case_t genders_isnode_corner_case_t;
-typedef genders_handle_pointer_corner_case_t genders_isattr_corner_case_t;
+typedef genders_handle_string_corner_case_t genders_isnode_corner_case_t;
+typedef genders_handle_string_corner_case_t genders_isattr_corner_case_t;
 
 typedef struct {
   int num;
   genders_handle_type_t param1;
-  genders_pointer_type_t param2;
-  genders_pointer_type_t param3;
+  genders_string_type_t param2;
+  genders_string_type_t param3;
   int expected_return_value;
   int expected_errnum;
 } genders_isattrval_corner_case_t;
 
-typedef genders_handle_pointer_corner_case_t genders_index_attrvals_corner_case_t;
+typedef genders_handle_string_corner_case_t genders_index_attrvals_corner_case_t;
 
 typedef struct {
   int num;
   genders_handle_type_t param1;
   genders_pointer_type_t param2;
   genders_length_type_t param3;
-  genders_pointer_type_t param4;
+  genders_string_type_t param4;
   int expected_return_value;
   int expected_errnum;
 } genders_query_corner_case_t;
@@ -177,8 +185,8 @@ typedef struct {
 typedef struct {
   int num;
   genders_handle_type_t param1;
-  genders_pointer_type_t param2;
-  genders_pointer_type_t param3;
+  genders_string_type_t param2;
+  genders_string_type_t param3;
   int expected_return_value;
   int expected_errnum;
 } genders_testquery_corner_case_t;
@@ -199,6 +207,13 @@ typedef struct {
   int expected_errnum;
 } genders_set_errnum_corner_case_t;
 
+typedef struct {
+  int num;
+  genders_handle_type_t param1;
+  genders_pointer_type_t expected_return_value;
+  int expected_errnum;
+} genders_copy_corner_case_t;
+
 extern genders_handle_destroy_corner_case_t genders_handle_destroy_corner_case_tests[];
 extern genders_load_data_corner_case_t genders_load_data_corner_case_tests[];
 extern genders_errnum_corner_case_t genders_errnum_corner_case_tests[];
@@ -234,6 +249,7 @@ extern genders_query_corner_case_t genders_query_corner_case_tests[];
 extern genders_testquery_corner_case_t genders_testquery_corner_case_tests[];
 extern genders_parse_corner_case_t genders_parse_corner_case_tests[];
 extern genders_set_errnum_corner_case_t genders_set_errnum_corner_case_tests[];
+extern genders_copy_corner_case_t genders_copy_corner_case_tests[];
 
 typedef int (*GendersCornerCaseFunc)(int);
 
@@ -273,5 +289,6 @@ int genders_query_corner_case(int verbose);
 int genders_testquery_corner_case(int verbose);
 int genders_parse_corner_case(int verbose);
 int genders_set_errnum_corner_case(int verbose);
+int genders_copy_corner_case(int verbose);
 
 #endif /* _GENDERS_TEST_CORNER_CASE_H */
