@@ -1,5 +1,5 @@
 /*****************************************************************************\
- *  $Id: genders_test_database.c,v 1.9 2010/02/02 00:04:34 chu11 Exp $
+ *  $Id: genders_test_database.c,v 1.9 2010-02-02 00:04:34 chu11 Exp $
  *****************************************************************************
  *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2001-2007 The Regents of the University of California.
@@ -74,6 +74,10 @@ genders_parse_error_database_t genders_parse_error_databases[] =
     {"testdatabases/genders.parse_error_duplicate_attr_same_line_attr_with_val", 1},
     {"testdatabases/genders.parse_error_duplicate_attr_same_line_attr_with_val_comma", 1},
     {"testdatabases/genders.parse_error_duplicate_attr_same_line_attr_with_val_hostrange", 1},
+    {"testdatabases/genders.parse_error_duplicate_attr_not_last_in_line", 1},
+    {"testdatabases/genders.parse_error_no_attr_before_equal_sign", 6},
+    {"testdatabases/genders.parse_error_no_value_after_equal_sign", 6},
+    {"testdatabases/genders.parse_error_errant_commas", 5},
     {"testdatabases/genders.parse_error_all", 8},
     {NULL, -1},
   };
@@ -229,6 +233,12 @@ genders_database_data_t genders_database_data_base =
 	{"node1", "node2", NULL},
 	2,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", NULL},
+	2,
+      },
       {
 	"attr1",
 	NULL,
@@ -248,7 +258,7 @@ genders_database_data_t genders_database_data_base =
 	2,
       },
     },
-    4,
+    5,
   };
 
 genders_database_t genders_database_base = 
@@ -483,6 +493,12 @@ genders_database_data_t genders_database_data_test_1 =
 	{"node1", "node2", "node3", "node4", NULL},
 	4,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", "node3", "node4", NULL},
+	4,
+      },
       {
 	"attr1",
 	NULL,
@@ -520,7 +536,7 @@ genders_database_data_t genders_database_data_test_1 =
 	4,
       },
     },
-    7,
+    8,
   };
 
 genders_database_t genders_database_test_1 =
@@ -597,6 +613,12 @@ genders_database_data_t genders_database_data_test_2 =
 	{"node1", "node2", "node3", "node4", NULL},
 	4,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", "node3", "node4", NULL},
+	4,
+      },
       {
 	"attr1",
 	NULL,
@@ -670,7 +692,7 @@ genders_database_data_t genders_database_data_test_2 =
 	2,
       },
     },
-    13,
+    14,
   };
 
 genders_database_t genders_database_test_2 =
@@ -751,6 +773,12 @@ genders_database_data_t genders_database_data_test_3 =
 	{"node1", "node2", "node3", "node4", NULL},
 	4,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", "node3", "node4", NULL},
+	4,
+      },
       {
 	"attr1",
 	NULL,
@@ -896,7 +924,7 @@ genders_database_data_t genders_database_data_test_3 =
 	1,
       },
     },
-    25,
+    26,
   };
 
 genders_database_t genders_database_test_3 =
@@ -982,13 +1010,18 @@ genders_database_data_t genders_database_data_equal_sign_in_value =
     },
     2,
     {
-
       {
 	NULL,
 	NULL,
 	{"node1", "node2", NULL},
 	2,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", NULL},
+	2,
+      },
       {
 	"attr1",
 	NULL,
@@ -1015,7 +1048,7 @@ genders_database_data_t genders_database_data_equal_sign_in_value =
 	1,
       },
     },
-    5,
+    6,
   };
 
 genders_database_t genders_database_equal_sign_in_value =
@@ -1058,8 +1091,14 @@ genders_database_data_t genders_database_data_nodes_only_one =
 	{"node1", NULL},
 	1,
       },
+      {
+	"",
+	"",
+	{"node1", NULL},
+	1,
+      },
     },
-    1,
+    2,
   };
 
 genders_database_t genders_database_nodes_only_one = 
@@ -1102,8 +1141,14 @@ genders_database_data_t genders_database_data_nodes_only_many =
 	{"node1", "node2", "node3", "node4", "node5", NULL},
 	5,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", "node3", "node4", "node5", NULL},
+	5,
+      },
     },
-    1,
+    2,
   };
 
 genders_database_t genders_database_nodes_only_many = 
@@ -1152,6 +1197,12 @@ genders_database_data_t genders_database_data_nodes_and_attrs_only =
 	{"node1", "node2", NULL},
 	2,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", NULL},
+	2,
+      },
       {
 	"attr1",
 	NULL,
@@ -1165,7 +1216,7 @@ genders_database_data_t genders_database_data_nodes_and_attrs_only =
 	2,
       },
     },
-    3,
+    4,
   };
 
 genders_database_t genders_database_nodes_and_attrs_only = 
@@ -1226,6 +1277,12 @@ genders_database_data_t genders_database_data_subst_escape_char =
 	{"node1", "node2", NULL},
 	2,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", NULL},
+	2,
+      },
       {
 	"attr1",
 	NULL,
@@ -1281,7 +1338,7 @@ genders_database_data_t genders_database_data_subst_escape_char =
 	2,
       },
     },
-    10,
+    11,
   };
 
 genders_database_t genders_database_subst_escape_char = 
@@ -1331,6 +1388,12 @@ genders_database_data_t genders_database_data_subst_nodename =
 	{"node1", "node2", NULL},
 	2,
       },
+      {
+	"",
+	"",
+	{"node1", "node2", NULL},
+	2,
+      },
       {
 	"attr1",
 	NULL,
@@ -1368,7 +1431,7 @@ genders_database_data_t genders_database_data_subst_nodename =
 	1,
       },
     },
-    7,
+    8,
   };
 
 genders_database_t genders_database_subst_nodename = 
@@ -1423,6 +1486,12 @@ genders_database_data_t genders_database_data_large_attr =
 	{"node1", NULL},
 	1,
       },
+      {
+	"",
+	"",
+	{"node1", NULL},
+	1,
+      },
       {
 	"abcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyzabcdefghijklmnopqrstuvwxyz",
 	NULL,
@@ -1430,7 +1499,7 @@ genders_database_data_t genders_database_data_large_attr =
 	1,
       },
     },
-    2,
+    3,
   };
 
 genders_database_t genders_database_large_attr = 
@@ -1473,6 +1542,12 @@ genders_database_data_t genders_database_data_large_val =
 	{"node1", NULL},
 	1,
       },
+      {
+	"",
+	"",
+	{"node1", NULL},
+	1,
+      },
       {
 	"attr1",
 	NULL,
@@ -1486,7 +1561,7 @@ genders_database_data_t genders_database_data_large_val =
 	1,
       },
     },
-    3,
+    4,
   };
 
 genders_database_t genders_database_large_val = 
